using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;

namespace Game
{
	/// <summary>
	/// Summary description for Form1.
	/// </summary>
	public class Game : System.Windows.Forms.Form
	{
		private System.Windows.Forms.Timer mainTimer;
		private System.ComponentModel.IContainer components;

		private Mouse_State mouse_state = new Mouse_State();

		private Resources resources;
		private Screen screen;

		private int lastTick = 0;

		public Game()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			// Enable double-buffering to avoid flicker
			SetStyle( ControlStyles.UserPaint, true );
			SetStyle( ControlStyles.AllPaintingInWmPaint, true );
			SetStyle( ControlStyles.DoubleBuffer, true );

			// Init the game
			resources = new Resources( CreateGraphics() );
			Switch_State( New_Screen_State.Menu );
		}

		private void Switch_State( New_Screen_State new_state )
		{
			if ( new_state == New_Screen_State.None )
				return;

			switch ( new_state )
			{
				case New_Screen_State.Menu:
					screen = new Menu_Screen( resources );
					break;

				case New_Screen_State.Game_1_Plr:
					screen = new Game_Screen( resources, 1 );
					break;

				case New_Screen_State.Game_2_Plr:
					screen = new Game_Screen( resources, 2 );
					break;

				case New_Screen_State.Game_3_Plr:
					screen = new Game_Screen( resources, 3 );
					break;

				case New_Screen_State.Game_4_Plr:
					screen = new Game_Screen( resources, 4 );
					break;
			}
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.components = new System.ComponentModel.Container();
			this.mainTimer = new System.Windows.Forms.Timer(this.components);
			// 
			// mainTimer
			// 
			this.mainTimer.Enabled = true;
			this.mainTimer.Interval = 30;
			this.mainTimer.Tick += new System.EventHandler(this.mainTimer_Tick);
			// 
			// Game
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(640, 480);
			this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedSingle;
			this.MaximizeBox = false;
			this.MinimizeBox = false;
			this.Name = "Game";
			this.Text = "Radox";
			this.Resize += new System.EventHandler(this.Form1_Resize);
			this.MouseDown += new System.Windows.Forms.MouseEventHandler(this.Game_MouseDown);
			this.Load += new System.EventHandler(this.Form1_Load);
			this.MouseUp += new System.Windows.Forms.MouseEventHandler(this.Game_MouseUp);
			this.KeyUp += new System.Windows.Forms.KeyEventHandler(this.Game_KeyUp);
			this.MouseMove += new System.Windows.Forms.MouseEventHandler(this.Game_MouseMove);

		}
		#endregion

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main() 
		{
			Application.Run(new Game());
		}

		protected override void OnPaint(PaintEventArgs e)
		{
//			base.OnPaint( e );

			// Calculate the FPS
			int fps = 0;
			int now = Environment.TickCount;
			if ( lastTick != 0 && now > lastTick )
			{
				int elapsed = now - lastTick;
				if ( elapsed > 0 )
					fps = 1000 / elapsed;
			}
			lastTick = now;

			// Render the screen
			screen.Render( e.Graphics, ClientRectangle.Width, ClientRectangle.Height );

			// Display the FPS
			Font f = new Font( "Courier New", 20 );
//			e.Graphics.DrawString( "FPS: " + fps, f, Brushes.White, 10, 10 );
		}

		protected override void OnPaintBackground(PaintEventArgs pevent)
		{
//			base.OnPaintBackground (pevent);
		}


		private void mainTimer_Tick(object sender, System.EventArgs e)
		{
			Invalidate();

			Switch_State( screen.Update( mouse_state ) );
			mouse_state.Update();
		}

		private void Form1_Load(object sender, System.EventArgs e)
		{
		}

		private void Form1_Resize(object sender, System.EventArgs e)
		{
			Invalidate();
		}

		private void Game_MouseMove(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			mouse_state.x = e.X;
			mouse_state.y = e.Y;
		}

		private void Game_MouseDown(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			switch ( e.Button )
			{
				case MouseButtons.Left:		mouse_state.Set_Key( Mouse_Key.Left, true ); break;
				case MouseButtons.Right:	mouse_state.Set_Key( Mouse_Key.Right, true ); break;
				case MouseButtons.Middle:	mouse_state.Set_Key( Mouse_Key.Middle, true ); break;
			}
		}

		private void Game_MouseUp(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			switch ( e.Button )
			{
				case MouseButtons.Left:		mouse_state.Set_Key( Mouse_Key.Left, false ); break;
				case MouseButtons.Right:	mouse_state.Set_Key( Mouse_Key.Right, false ); break;
				case MouseButtons.Middle:	mouse_state.Set_Key( Mouse_Key.Middle, false ); break;
			}
		}

		private void Game_KeyUp(object sender, System.Windows.Forms.KeyEventArgs e)
		{
			screen.Key_Pressed( e.KeyCode );
		}
	}
}
