using System;
using System.Drawing;
using System.Drawing.Imaging;

namespace Game
{
	/// <summary>
	/// Summary description for Menu_Screen.
	/// </summary>
	public class Menu_Screen : Screen
	{
		public static String version = "v1.1";

		private float ang = 0.0f;

		private Gfx_Button[] buttons =
		{
			new Gfx_Button( "1 player", 40, new Rectangle( 100, 150, 440, 70 ), true ),
			new Gfx_Button( "2 players", 40, new Rectangle( 100, 230, 440, 70 ), true ),
			new Gfx_Button( "3 players", 40, new Rectangle( 100, 310, 440, 70 ), true ),
			new Gfx_Button( "4 players", 40, new Rectangle( 100, 390, 440, 70 ), true ),
		};

		private int mouse_on_over = -1;
		private New_Screen_State selected = New_Screen_State.None;

		public Menu_Screen( Resources res )
		{
		}
		#region Screen Members

		public New_Screen_State Update( Mouse_State mouse )
		{
			ang += 0.1f;

			if ( selected == New_Screen_State.None )
			{
				mouse_on_over = -1;
				for ( int i = 0; i < buttons.Length; ++i )
				{
					if ( buttons[ i ].Is_In_Button( mouse.x, mouse.y ) )
						mouse_on_over = i;
				}
			}

			if ( mouse.Is_Pressed( Mouse_Key.Left ) && mouse_on_over >= 0 )
			{
				selected = (New_Screen_State) ( (int)New_Screen_State.Game_1_Plr + mouse_on_over );
			}

			if ( mouse.Is_Released( Mouse_Key.Left ) )
				return selected;
			return New_Screen_State.None;
		}

		// Notifies of a keypress
		public void Key_Pressed( System.Windows.Forms.Keys key )
		{
		}

		public void Render(System.Drawing.Graphics g, int scr_w, int scr_h)
		{
//			g.FillRectangle( Brushes.Black, 0, 0, scr_w, scr_h );

			float radius = 21;
			float size = 30;
			float draw_size = size + radius * 2;
			float draw_size2 = draw_size / 2.0f;
			int x_cnt = (int) ( scr_w / size ) + 2;
			int y_cnt = (int) ( scr_h / size ) + 2;

			// Draw the background
			Brush brush = new SolidBrush( Color.FromArgb( 50, 155, 255, 155 ) );
			for ( int y = -1; y < y_cnt; ++y )
			{
				for ( int x = -1; x < x_cnt; ++x )
				{
					float xang = (float) Math.Sin( ang + y * 0.5f );
					float yang = (float) Math.Sin( ang + x * 0.4f );
					float a = ang + x * (float)Math.Cos( xang ) * 0.1f + y * (float) Math.Sin( yang ) * 0.1f;
					float m1 = (float) ( Math.Sin( a * 0.023f ) * Math.Cos( a * 0.003f ) );
					float m2 = (float) ( Math.Cos( a * 0.043f ) * Math.Sin( a * 0.011f ) );
					float sx = (float) ( Math.Cos( a * m1 ) * Math.Sin( a * 1.234f ) );
					float sy = (float) ( Math.Sin( a * m2 ) * Math.Cos( a * 0.925f ) );
					sx = x * size + sx * radius;
					sy = y * size + sy * radius;

					g.FillRectangle( brush, sx - draw_size2, sy - draw_size2, draw_size, draw_size ); 
				}
			}

			// Draw the buttons
			for ( int i = 0; i < buttons.Length; ++i )
			{
				buttons[ i ].Draw( g, mouse_on_over == i );
			}

			// Draw the title text
			if ( true )
			{
				Font f = new Font( "Comic Sans MS", 100 );
				Brush title_brush = new SolidBrush( Color.FromArgb( 40, 155, 15, 15 ) );
				Brush title_brush_b = new SolidBrush( Color.FromArgb( 150, 195, 35, 35 ) );

				int x = (int) ( (float) Math.Cos( ang * 1.1f ) * 8.0f );
				int y = (int) ( (float) Math.Sin( ang * 1.1f ) * 8.0f );

				Resources.Write_Center( g, title_brush_b, f, "Radox", scr_w / 2, 80 );
				Resources.Write_Center( g, title_brush, f, "Radox", scr_w / 2 - x, 80 + y );
				Resources.Write_Center( g, title_brush, f, "Radox", scr_w / 2 - x, 80 - y );
				Resources.Write_Center( g, title_brush, f, "Radox", scr_w / 2 + x, 80 + y );
				Resources.Write_Center( g, title_brush, f, "Radox", scr_w / 2 + x, 80 - y );
			}

			// Draw the version and author
			if ( true )
			{
				Font f = new Font( "Comic Sans MS", 8, FontStyle.Bold );
				Resources.Write_Shade( g, f, "Author: Jere Sanisalo    Version: " + version, 2, scr_h - 18 );
			}
		}

		#endregion
	}
}
